import re

def split_sentences(text):
    placeholder_map = {}

    def replace_ip(match):
        key = f"<IP{len(placeholder_map)}>"
        placeholder_map[key] = match.group()
        return key

    text = re.sub(r'\b(?:\d{1,3}\.){3}\d{1,3}\b', replace_ip, text)

    sentences = re.split(r'(?<=[.!?])\s+(?=[A-ZА-ЯЎҚҒЁ])', text.strip())

    restored = [re.sub(r'<IP\d+>', lambda m: placeholder_map[m.group()], s) for s in sentences]

    return [s.strip() for s in restored if s.strip()]


def split_words(text, remove_ip=True, remove_email=True, remove_url=True, remove_emoji=True, remove_punct=True):
    if remove_ip:
        text = re.sub(r'\b(?:\d{1,3}\.){3}\d{1,3}\b', '', text)

    if remove_email:
        text = re.sub(r'\b[\w\.-]+@[\w\.-]+\.\w+\b', '', text)

    if remove_url:
        text = re.sub(r'https?://\S+|www\.\S+', '', text)

    if remove_emoji:
        emoji_pattern = re.compile(
            "["
            "\U0001F600-\U0001F64F"
            "\U0001F300-\U0001F5FF"
            "\U0001F680-\U0001F6FF"
            "\U0001F1E0-\U0001F1FF"
            "]+", flags=re.UNICODE)
        text = emoji_pattern.sub(r'', text)

    if remove_punct:
        text = re.sub(r'[^\w\sʼ’ʻ\-]', '', text)

    tokens = re.findall(r'\b[\wʼ’ʻ\-]+\b', text)

    cleaned = []
    for token in tokens:
        if re.fullmatch(r'(milodiy\s*)?\d{4}[- ]?(yil|y\.?)?(dan|ga|da|ni|ning)?', token.lower()):
            cleaned.append(token)
        elif token.isdigit():
            continue
        elif re.fullmatch(r'[a-zA-Zа-яА-ЯёЁʼ’ʻ\-]+', token):
            cleaned.append(token)

    return cleaned



import re

def solid_sign(text):

    text = re.sub(r"([oOgG])[`'‘’´]", r"\1<<SAFE>>", text)

    words_with_apostrophe = re.findall(r"\b\w*[`'‘’´]\w*\b", text)

    cleaned_words = []
    for word in words_with_apostrophe:
        cleaned = re.sub(r"[`'‘’´]", "’", word)
        cleaned_words.append(cleaned)

    return cleaned_words



import re

def clean_text(text):
    apostrophes = ["'", "`", "‘", "’", "´"]

    pattern = r"[`'‘’´]"
    text = re.sub(pattern, "’", text)

    for a in apostrophes:
        text = text.replace(f"o{a}", "o‘")
        text = text.replace(f"g{a}", "g‘")
        text = text.replace(f"O{a}", "O‘")
        text = text.replace(f"G{a}", "G‘")

    return text


import requests
import zipfile
import io
import os
import shutil

def download_folder(folder_name):
    repo_url = "https://github.com/JasurOmanov/Manbalar"
    repo_zip_url = repo_url.rstrip("/") + "/archive/refs/heads/main.zip"
    headers = {
        "User-Agent": "Mozilla/5.0"
    }

    try:
        print(f"'{folder_name}' papkasi yuklab olinmoqda...")
        response = requests.get(repo_zip_url, headers=headers)
        response.raise_for_status()
    except requests.RequestException as e:
        print(f"Yuklab olishda xatolik: {e}")
        return

    try:
        with zipfile.ZipFile(io.BytesIO(response.content)) as z:
            main_dir = z.namelist()[0].split('/')[0]
            extract_path = f"{main_dir}/{folder_name}"
            found = False
            extract_to = f"temp_extract_{folder_name}"

            if os.path.exists(extract_to):
                shutil.rmtree(extract_to)
            os.makedirs(extract_to)

            for file in z.namelist():
                if file.startswith(extract_path + "/") and not file.endswith("/"):
                    found = True
                    z.extract(file, extract_to)

            if not found:
                print(f"'{folder_name}' papkasi topilmadi.")
                shutil.rmtree(extract_to)
                return

            if not os.path.exists("Manba"):
                os.makedirs("Manba")

            final_path = os.path.join("Manba", folder_name)
            if os.path.exists(final_path):
                shutil.rmtree(final_path)
            shutil.move(os.path.join(extract_to, main_dir, folder_name), final_path)
            shutil.rmtree(extract_to)
            print(f"'{folder_name}' papkasi 'Manba/' ichiga muvaffaqiyatli yuklandi.")

    except zipfile.BadZipFile:
        print("ZIP faylni ochishda xatolik.")
    except Exception as e:
        print(f"Faylni ajratishda xatolik: {e}")

def Book():
    download_folder("Uzb_kitoblar")

def News():
    download_folder("Yangiliklar")



from UzMorphAnalyser import UzMorphAnalyser

# Avval obyekt yaratish kerak
analyser = UzMorphAnalyser()
def stem_word(word):
    # So‘zni lemmatizatsiya qilish
    a=analyser.stem(word)
    return a




def syllables(text):
    tokens = processing(text)
    sylls = list()
    for token in tokens:
        count = 0
        vowels = set("AaEeUuOoIiАаОоУуИиЯяЕеЁёЮюЭэЎў")
        syll = list()
        start = 0
        for letter in token:
            for i in range(65, 91):
                if ord(letter) == i:
                    count += 1
            for i in range(1040, 1072):
                if ord(letter) == i:
                    count += 1
        if count > 1:
            sylls.append(token)
            continue
        count = 0
        for letter in token:
            if letter in vowels:
                count += 1
        if count == 1:
            sylls.append(token)
            continue
        for i in range(2, len(token)):
            if token[i] in vowels and token[i - 1] not in vowels:
                w = token[start: i - 1]
                if len(w) != 0:
                    syll.append(w)
                    start = i - 1
            if token[i] in vowels and token[i - 1] in vowels:
                w = token[start: i]
                if len(w) != 0:
                    syll.append(w)
                    start = i
        w = token[start:len(token)]
        syll.append(w)
        count = 0
        for i in syll[0]:
            if i in vowels:
                count += 1
        if count == 0:
            s = syll[1]
            syll[1] = syll[0]+s
            syll[0] = ""
        if count == 2:
            s = syll[0]
            syll.remove(syll[0])
            temp = list()
            start = 0
            for i in range(1, len(s)):
                if s[i] in vowels and s[i - 1] in vowels:
                    w = s[start: i]
                    if len(w) != 0:
                        temp.append(w)
                        start = i
                w = s[start:len(s)]
                temp.append(w)
            syll = temp + syll
        while len(''.join(syll)) != len(token):
            d = ''
            for i in range(1, len(syll)):
                s = ''.join(syll)
                if syll[i] == syll[i-1] and s != token:
                    d = syll[i]
            syll.remove(d)
        for i in range(1, len(syll)):
            if syll[i][0] == 'h' or syll[i][0] == chr(8216):
                if syll[i-1][len(syll[i-1])-1] in ['s', 'c'] or syll[i][0] == chr(8216):
                    s = syll[i]
                    syll[i] = syll[i-1][len(syll[i-1])-1] + s
                    s = syll[i-1]
                    syll[i-1] = s[0:len(s)-1]
        for i in range(1, len(syll)):
            if syll[i][0] == 'g' and syll[i][1] != chr(8216) and len(syll[i-1]) > 0:
                if syll[i-1][len(syll[i-1])-1] == 'n':
                    s = syll[i]
                    syll[i] = syll[i-1][len(syll[i-1])-1] + s
                    s = syll[i-1]
                    syll[i-1] = s[0:len(s)-1]
        str = ""
        for w in syll:
            if len(w) > 1 and w[len(w)-1] == '-':
                w = w[0:len(w)-1]
            if len(w) > 1 and w[0] == '-':
                w = w[1:len(w)]
            if w != '':
                str += w
                if not w.__contains__('-'):
                    str += '-'
        sylls.append(str[0:len(str)-1])
    return sylls
def processing(text):
    text = str(text)
    text = text.replace("`", "'")
    text = text.replace("O'", "O‘").replace("o'", "o‘").replace("G'", "G‘").replace("g'", "g‘").replace("'", "’")
    return text.split()
def hyphenation(text):
    word = text
    syllable = ' '.join(syllables(text))
    begin = end = ''
    if word.__contains__('-'):
        for i in range(0, len(word)):
            if word[i] == '-':
                begin = word[i - 1]
                end = word[i + 1]
    lines = list()
    if not syllable.__contains__('-'):
        lines.append(syllable)
    tokens = syllable.split('-')
    count = len(tokens) - 1
    for j in range(1, len(tokens)):
        if len(tokens[j]) != 1:
            w = ''
            for i in range(0, j):
                w += tokens[i]
            if len(tokens[j - 1]) != 1 and not tokens[j - 1].__contains__(chr(8217)):
                w += '-'
            for i in range(j, len(tokens)):
                if len(tokens[i]):
                    w += tokens[i]
            if begin != end != '' and len(end):
                for i in range(0, len(w) - 1):
                    if w[i] == begin and w[i + 1] == end:
                        w = w[0:i + 1] + '-' + w[i + 1:len(w)]
                        break
            if w != word or count == 1 or word.__contains__('-') and w not in lines:
                lines.append(w)
    return lines
def count(text):
    tokens = syllables(text)
    count = 0
    for token in tokens:
        syll = token.split('-')
        count += len(syll)
    return count



import os

def stop_word():

    path = os.path.join(os.path.dirname(__file__), 'data', 'stopwords_uz.txt')
    with open(path, 'r', encoding='utf-8') as f:
        return f.read().splitlines()

def clean_stop_words(text):
    stopwords = stop_word()
    if isinstance(text, str):
        str_list = text.split()
    else:
        str_list = text
    return [t for t in str_list if t not in stopwords]



from setuptools import setup, find_packages

setup(
    name="uznltk",
    version="0.0.5",
    author="Ulugbek Salaev",
    author_email="ulugbek0302@gmail.com",
    description="The Uzbek Natural Language Toolkit (NLTK) is a Python package for natural language processing.",
    long_description=open("README.md", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/UlugbekSalaev/uznltk",
    keywords=['nltk', 'morphology', 'uzbek-language', 'pos tagging', 'morphological tagging'],
    project_urls={
        "Bug Tracker": "https://github.com/UlugbekSalaev/uznltk/issues",
    },
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.7",
    install_requires=[
        "requests",
        "UzMorphAnalyser"
    ],
    include_package_data=True,
    license="MIT",
)



#from .tokenizer import split_sentences, split_words
from .stop_words import stop_word, clean_stop_words
from .download import Book, News
from .lemmatizer import lemmatize
from .stem import stem_word
from .clean import clean_text
from .ssign import solid_sign
from .syllable import syllables


from UzMorphAnalyser import UzMorphAnalyser

# Avval obyekt yaratish kerak
analyser = UzMorphAnalyser()
def lemmatize(word):
    # So‘zni lemmatizatsiya qilish
    a=analyser.lemmatize(word)
    return a

